require 'rest-client'
require 'json'
require 'uri'
require 'pp'
require 'logger'


class OpenbookingClientRuby

	def initialize(uri, logger=Logger.new(STDOUT))
		RestClient.proxy = ENV["PROXY"] unless ENV["PROXY"].nil?

		@logger = logger
		@opts = {}
		ob_url = URI::parse(uri)

		if ob_url.userinfo
			auth = ob_url.userinfo.split(':')
			@opts[:user] = auth[0]
			@opts[:pass] = auth[1]
		end

		@opts[:url] = "#{ob_url.scheme}://#{ob_url.host}:#{ob_url.port}"
	end

	def auth
		@logger.debug 'Get OpenBooking Token'
		response = post '/authenticate', {
				:user => @opts[:user],
				:pass => @opts[:pass]
		}

		@logger.debug "Got OpenBooking Token, ID: #{response[:access_token]}"
		@token = response[:access_token]
	end

	def auth_as(username)
		auth
		all_users = get "/users/all?user=#{username}&fields=id,user,name,group"
		user = all_users[0]
		token = post("/tokens/"+user[:id], {scopes: ["master"]})
		use_token(token[:access_token])
	end

	def use_token(token)
		@logger.debug "Using Token #{token}"
		@token = token
	end

	def get(path, params = {}, headers = {})
		qs = URI.encode_www_form params
		uri = "#{@opts[:url]}#{path}?#{qs}"
		@logger.debug "GET #{uri}"
		headers[:accept] = "json"
		headers[:authorization] = "Bearer #{@token}"
		JSON.parse RestClient.send('get', uri, headers), :symbolize_names => true
	rescue Exception => e
		puts "Couldn't request get: #{path} #{params}"
		pp e
	end

	def post(path, params = {})
		uri = "#{@opts[:url]}#{path}"

		headers = { :accept => :json, :content_type => :json }
		headers[:authorization] = "Bearer #{@token}"

		data = params.to_json
		if !data
			data = {}.to_json
		end
		JSON.parse RestClient.send('post', uri, data, headers), :symbolize_names => true
	rescue Exception => e
		@logger.fatal "Couln't request post: #{path} #{params}"
		pp e
	end

	def put(path, params = {})
		uri = "#{@opts[:url]}#{path}"
		headers = { :accept => :json, :content_type => :json }
		data = params.to_json
		headers[:authorization] = "Bearer #{@token}"
		JSON.parse RestClient.send('put', uri, data, headers), :symbolize_names => true
	rescue Exception => e
		@logger.fatal "Couln't request put: #{path} #{params}"
		pp e
	end

	def patch(path, params = {})
		uri = "#{@opts[:url]}#{path}"
		headers = { :accept => :json, :content_type => :json }
		data = params.to_json
		headers[:authorization] = "Bearer #{@token}"
		JSON.parse RestClient.send('patch', uri, data, headers), :symbolize_names => true
	rescue Exception => e
		@logger.fatal "Couln't request patch: #{path} #{params}"
		pp e
	end


	def del(path, params = {})
		qs = URI.encode_www_form params
		uri = "#{@opts[:url]}#{path}?#{qs}"
		headers = { :accept => :json }
		headers[:authorization] = "Bearer #{@token}"
		JSON.parse RestClient.send('delete', uri, headers), :symbolize_names => true
	rescue Exception => e
		@logger.fatal "Couldn't request delete: #{path} #{params}"
		pp e
	end

	def close
		del "/token/#{@token}"
		remove_instance_variable :@token
	end


end

